@@ -12,6 +12,7 @@
 (require 'ai-code-input)
 (require 'ai-code-prompt-mode)
 (require 'thingatpt)
+(require 'which-func)
 
 (declare-function ai-code--insert-prompt "ai-code-prompt-mode" (prompt-text))
 (declare-function ai-code--get-context-files-string "ai-code-input")
@@ -579,30 +580,54 @@ If no such buffer is found, report a user-error."
     (unless has-test-buffer
       (user-error "No test file found in current windows. Please open a test file first"))))
 
+(defconst ai-code--tdd-test-pattern-instruction
+  "\nFollow the test-code pattern in the current project. Write the test-code in the test-file. If the test-file does not exist, create it using the same test-filename pattern used in this repository."
+  "Instruction appended to TDD prompts to enforce the project's test pattern.")
+
 (defun ai-code--tdd-red-stage (function-name)
   "Handle the Red stage of TDD for FUNCTION-NAME: Write a failing test."
-  (if (and (region-active-p) (not (derived-mode-p 'prog-mode)))
-      ;; If there is a selected region, and it is not a prog-mode derived buffer,
-      ;; assume this is the information / exception of failed test.
-      ;; Use it as initial prompt to fix the source code.
-      (let* ((failure-info (buffer-substring-no-properties (region-beginning) (region-end)))
-             (prompt (ai-code-read-string "Confirm test failure to fix: " failure-info))
+  (let ((test-pattern-instruction ai-code--tdd-test-pattern-instruction))
+    (if (and (region-active-p) (not (derived-mode-p 'prog-mode)))
+        ;; If there is a selected region, and it is not a prog-mode derived buffer,
+        ;; assume this is the information / exception of failed test.
+        ;; Use it as initial prompt to fix the source code.
+        (let* ((failure-info (buffer-substring-no-properties (region-beginning) (region-end)))
+               (prompt (ai-code-read-string "Confirm test failure to fix: " failure-info))
+               (file-info (ai-code--get-context-files-string))
+               (tdd-instructions (format "Fix the code to resolve the following error:\n%s%s%s"
+                                         prompt
+                                         file-info
+                                         test-pattern-instruction)))
+          (ai-code--insert-prompt tdd-instructions))
+      ;; Original path: write a failing test
+      (ai-code--ensure-test-buffer-visible)
+      (let* ((feature-desc (ai-code-read-string
+                            (if function-name
+                                (format "Describe the feature to test for '%s': " function-name)
+                              "Describe the feature to test: ") "Implement test functions using test cases described in the comments."))
              (file-info (ai-code--get-context-files-string))
-             (tdd-instructions (format "Fix the code to resolve the following error:\n%s%s" prompt file-info)))
-        (ai-code--insert-prompt tdd-instructions))
-    ;; Original path: write a failing test
-    ;; DONE: at least one buffer in current window should be a test
-    ;; file (contains 'test' in name, case insensitive), otherwise
-    ;; report error and exit this function
-    (ai-code--ensure-test-buffer-visible)
-    (let* ((feature-desc (ai-code-read-string
-                          (if function-name
-                              (format "Describe the feature to test for '%s': " function-name)
-                            "Describe the feature to test: ") "Implement test functions using test cases described in the comments."))
-           (file-info (ai-code--get-context-files-string))
-           (tdd-instructions
-            (format "%s%s\nFollow TDD principles - write only the test now, not the implementation. The test should fail when run because the functionality doesn't exist yet. Only update test file code." feature-desc file-info)))
-      (ai-code--insert-prompt tdd-instructions))))
+             (tdd-instructions
+              (format "%s%s\nFollow TDD principles - write only the test now, not the implementation. The test should fail when run because the functionality doesn't exist yet. Only update test file code.%s"
+                      feature-desc
+                      file-info
+                      test-pattern-instruction)))
+        (ai-code--insert-prompt tdd-instructions)))))
+
+(defun ai-code--tdd-red-green-stage (function-name)
+  "Handle the Red + Green stage for FUNCTION-NAME in one prompt."
+  ;; (ai-code--ensure-test-buffer-visible)
+  (let* ((feature-desc (ai-code-read-string
+                        (if function-name
+                            (format "Describe the feature to test for '%s': " function-name)
+                          "Describe the feature to test: ")
+                        "Implement test functions using test cases described in the comments."))
+         (file-info (ai-code--get-context-files-string))
+         (tdd-instructions
+          (format "%s%s\nFollow TDD principles - write the failing test first, then implement the minimal code to make it pass. Only update test and source code. Run the tests and follow up with the @test result (fix code if there is error).%s"
+                  feature-desc
+                  file-info
+                  ai-code--tdd-test-pattern-instruction)))
+    (ai-code--insert-prompt tdd-instructions)))
 
 (defun ai-code--tdd-green-stage (function-name)
   "Handle the Green stage of TDD for FUNCTION-NAME: Make the test pass.
@@ -625,6 +650,26 @@ to fix code."
                   implementation-desc file-info)))
     (ai-code--insert-prompt tdd-instructions)))
 
+(defun ai-code--run-test-ai-assisted ()
+  "Run tests by sending a prompt to AI with current context."
+  (let* ((is-dired (derived-mode-p 'dired-mode))
+         (function-name (unless is-dired (which-function)))
+         (file-info (unless is-dired (ai-code--get-context-files-string)))
+         (initial-input
+          (cond
+           (is-dired
+            (format "Run the tests for source code in directory '%s' using appropriate test runner."
+                    (dired-current-directory)))
+           (function-name
+            (format "Run the tests for the current function '%s' using appropriate test runner.%s"
+                    function-name
+                    file-info))
+           (t
+            (format "Run the tests for the current file using appropriate test runner.%s"
+                    file-info))))
+         (prompt (ai-code-read-string "Send to AI: " initial-input)))
+    (ai-code--insert-prompt prompt)))
+
 ;;;###autoload
 (defun ai-code-run-test ()
   "Run tests based on the current buffer's mode.
@@ -635,7 +680,9 @@ them if available."
    ((derived-mode-p 'python-mode)
     (if (fboundp 'python-pytest-popup)
         (python-pytest-popup)
-      (message "emacs-python-pytest package is required to run python test.")))
+      (progn
+        (message "emacs-python-pytest package is required to run python test.")
+        (ai-code--run-test-ai-assisted))))
    ((or (derived-mode-p 'js-mode)
         (derived-mode-p 'js-ts-mode)
         (derived-mode-p 'typescript-mode)
@@ -643,12 +690,14 @@ them if available."
         (derived-mode-p 'rjsx-mode))
     (if (fboundp 'jest-popup)
         (jest-popup)
-      (message "jest package is required to run js/ts test.")))
+      (progn
+        (message "jest package is required to run js/ts test.")
+        (ai-code--run-test-ai-assisted))))
    ((derived-mode-p 'emacs-lisp-mode)
     (require 'ert)
     (ert t))
    (t
-    (message "Test runner not supported for current mode: %s" major-mode))))
+    (ai-code--run-test-ai-assisted))))
 
 ;;;###autoload
 (defun ai-code-tdd-cycle ()
@@ -656,13 +705,15 @@ them if available."
 Helps users follow Kent Beck's TDD methodology with AI assistance.
 Works with both source code and test files that have been added to ai-code."
   (interactive)
+  ;; DONE: Append a new option to the tail: Red + Green. This allows users to complete both stages in one prompt. After that, AI should run the test, follow up with the @test result (eg. fix the code if there is error).
   (let* ((function-name (which-function))
          (cycle-stage (completing-read
                        "Select TDD stage: "
                        '("0. Run unit-tests"
                          "1. Red (Write failing test)"
                          "2. Green (Make test pass)"
-                         "3. Refactor (Improve code quality)")
+                         "3. Refactor (Improve code quality)"
+                         "4. Red + Green (One prompt)")
                        nil t))
          (stage-num (string-to-number (substring cycle-stage 0 1))))
     (cond
@@ -673,7 +724,9 @@ Works with both source code and test files that have been added to ai-code."
      ;; Green stage - make test pass
      ((= stage-num 2) (ai-code--tdd-green-stage function-name))
      ;; Refactor stage - call the main refactoring function in TDD mode
-     ((= stage-num 3) (ai-code-refactor-book-method t)))))
+     ((= stage-num 3) (ai-code-refactor-book-method t))
+     ;; Red + Green combined in one prompt
+     ((= stage-num 4) (ai-code--tdd-red-green-stage function-name)))))
 
 (provide 'ai-code-agile)
 ;;; ai-code-agile.el ends here
