@@ -1,8 +1,8 @@
-;;; ai-code.el --- Unified interface for AI coding CLI such as Codex, Copilot CLI, Opencode, Grok CLI, etc -*- lexical-binding: t; -*-
+;;; ai-code.el --- Unified interface for AI coding CLI such as Claude Code, Codex CLI, Gemini CLI, Copilot CLI, Opencode, Grok CLI, Aider CLI, etc -*- lexical-binding: t; -*-
 
 ;; Author: Kang Tu <tninja@gmail.com>
-;; Package-Version: 20260123.1525
-;; Package-Revision: 0f5178bef9fd
+;; Package-Version: 20260209.1524
+;; Package-Revision: a5143fce3d8e
 ;; Package-Requires: ((emacs "28.1") (transient "0.8.0") (magit "2.1.0"))
 ;; URL: https://github.com/tninja/ai-code-interface.el
 
@@ -11,37 +11,48 @@
 ;;; Commentary:
 ;; This package provides a uniform Emacs interface for various AI-assisted software
 ;; development CLI tools. Its purpose is to offer a consistent user experience
-;; across different AI backends while integrating seamlessly with AI-driven
-;; agile development workflows.
+;; across different AI backends, providing context-aware code actions, and integrating
+;; seamlessly with AI-driven agile development workflows.
 ;;
 ;; URL: https://github.com/tninja/ai-code-interface.el
 ;;
 ;; Supported AI coding CLIs include:
-;;   - Claude Code (claude-code.el)
+;;   - Claude Code
 ;;   - Gemini CLI
 ;;   - OpenAI Codex
 ;;   - GitHub Copilot CLI
 ;;   - Opencode
 ;;   - Grok CLI
 ;;   - Cursor CLI
+;;   - Kiro CLI
+;;   - CodeBuddy Code CLI
+;;   - Aider CLI
 ;;
 ;; Basic configuration example:
 ;;
 ;; (use-package ai-code
-;;   :straight (:host github :repo "tninja/ai-code-interface.el")
 ;;   :config
-;;   ;; use codex as backend, other options are 'gemini, 'github-copilot-cli, 'opencode, 'grok, 'claude-code-ide, 'claude-code, 'cursor
+;;   ;; use codex as backend, other options are 'gemini, 'github-copilot-cli, 'opencode, 'grok, 'claude-code-ide, 'claude-code-el, 'claude-code, 'cursor, 'kiro, 'codebuddy, 'aider
 ;;   (ai-code-set-backend 'codex) ;; set your preferred backend
 ;;   (global-set-key (kbd "C-c a") #'ai-code-menu)
-;;   (global-auto-revert-mode 1))
+;;   ;; Optional: Enable @ file completion in comments and AI sessions
+;;   (ai-code-prompt-filepath-completion-mode 1)
+;;   ;; Optional: Ask AI to run test after code changes, for a tighter build-test loop
+;;   (setq ai-code-auto-test-type 'test-after-change)
+;;   ;; Optional: In the AI session buffer (Evil normal state), SPC triggers the prompt entry UI
+;;   (with-eval-after-load 'evil (ai-code-backends-infra-evil-setup))
+;;   (global-auto-revert-mode 1)
+;;   (setq auto-revert-interval 1) ;; set to 1 second for faster update
+;;   )
 ;;
 ;; Key features:
 ;;   - Transient-driven Hub (C-c a) for all AI capabilities.
 ;;   - One key switching to different AI backend (C-c a s).
-;;   - Context-aware code actions (change code, implement TODOs, explain code).
-;;   - Agile development workflows (TDD cycle, refactoring navigator, review helper).
+;;   - Context-aware code actions (change code, implement TODOs, explain code, @ completion).
+;;   - Agile development workflows (TDD cycle, refactoring navigator, review helper, Build / Test feedback loop).
 ;;   - Seamless prompt management using Org-mode.
 ;;   - AI-assisted bash commands and productivity utilities.
+;;   - Multiple AI coding sessions management.
 ;;
 ;; Many features are ported from aider.el, making it a powerful alternative for
 ;; developers who wish to switch between modern AI coding CLIs while keeping
@@ -64,11 +75,14 @@
 (require 'ai-code-change)
 (require 'ai-code-discussion)
 (require 'ai-code-codex-cli)
+(require 'ai-code-aider-cli)
 (require 'ai-code-github-copilot-cli)
 (require 'ai-code-opencode)
 (require 'ai-code-grok-cli)
+(require 'ai-code-codebuddy-cli)
 (require 'ai-code-file)
 (require 'ai-code-ai)
+(require 'ai-code-notifications)
 
 ;; Forward declarations for dynamically defined backend functions
 (declare-function ai-code-cli-start "ai-code-backends")
@@ -79,6 +93,7 @@
 (declare-function ai-code-set-backend "ai-code-backends")
 (declare-function ai-code-select-backend "ai-code-backends")
 (declare-function ai-code-open-backend-config "ai-code-backends")
+(declare-function ai-code-open-backend-agent-file "ai-code-backends")
 (declare-function ai-code-upgrade-backend "ai-code-backends")
 (declare-function ai-code-backends-infra--session-buffer-p "ai-code-backends-infra" (buffer))
 
@@ -102,11 +117,60 @@ with a newline separator."
   :type '(choice (const nil) string)
   :group 'ai-code)
 
+;;;###autoload
 (defcustom ai-code-use-prompt-suffix t
   "When non-nil, append `ai-code-prompt-suffix` where supported."
   :type 'boolean
   :group 'ai-code)
 
+;;;###autoload
+(defcustom ai-code-test-after-code-change-suffix
+  "If any program code changes, run unit-tests and follow up on the test-result (fix code if there is an error)."
+  "User-provided prompt suffix for test-after-code-change."
+  :type '(choice (const nil) string)
+  :group 'ai-code)
+
+;;;###autoload
+(defvar ai-code-auto-test-suffix ai-code-test-after-code-change-suffix
+  "Default prompt suffix to request running tests after code changes.")
+
+(defun ai-code--test-after-code-change--resolve-tdd-suffix ()
+  "Return the TDD-style suffix for test-after-code-change prompts."
+  (let ((pattern (when (and (boundp 'ai-code--tdd-test-pattern-instruction)
+                            (stringp ai-code--tdd-test-pattern-instruction))
+                   ai-code--tdd-test-pattern-instruction)))
+    (concat "Follow TDD principles - write the failing test first, then implement the minimal code to make it pass. Only update test and source code. Run the tests and follow up with the test result (fix code if there is error)."
+            (or pattern ""))))
+
+(defun ai-code--test-after-code-change--set (symbol value)
+  "Set SYMBOL to VALUE and update related suffix behavior."
+  (set-default symbol value)
+  (set symbol value)
+  (pcase value
+    ('test-after-change
+     (setq ai-code-auto-test-suffix
+           ai-code-test-after-code-change-suffix))
+    ('tdd
+     (setq ai-code-auto-test-suffix
+           (ai-code--test-after-code-change--resolve-tdd-suffix)))
+    (_
+     (setq ai-code-auto-test-suffix nil))))
+
+(defun ai-code--apply-auto-test-type (value)
+  "Set `ai-code-auto-test-type` and refresh related suffix."
+  (setq ai-code-auto-test-type value)
+  (ai-code--test-after-code-change--set 'ai-code-auto-test-type value)
+  value)
+
+(defcustom ai-code-auto-test-type nil
+  "Select how prompts request tests after code changes."
+  :type '(choice (const :tag "Use test after code change prompt" test-after-change)
+                 (const :tag "Use TDD Red+Green prompt" tdd)
+                 (const :tag "Off" nil))
+  :set #'ai-code--test-after-code-change--set
+  :group 'ai-code)
+
+
 ;;;###autoload
 (defcustom ai-code-cli "claude"
   "The command-line AI tool to use for `ai-code-apply-prompt-on-current-file`."
@@ -130,27 +194,40 @@ with a newline separator."
 ;;;###autoload
 (defun ai-code-send-command (arg)
   "Read a prompt from the user and send it to the AI service.
-With a prefix argument (\\[universal-argument]), append the clipboard
-contents as context.
+With \\[universal-argument], append files and repo context.
+With \\[universal-argument] \\[universal-argument], also append clipboard context.
 ARG is the prefix argument."
+  ;; Prefix levels control whether files/repo and clipboard context are included,
+  ;; and the prompt label reflects the selected context.
   (interactive "P")
   (let* ((initial-input (when (use-region-p)
                           (string-trim-right
                            (buffer-substring-no-properties (region-beginning)
                                                            (region-end))
                            "\n")))
-         (clipboard-context (when arg (ai-code--get-clipboard-text)))
-         (prompt-label (if (and clipboard-context
-                                (string-match-p "\\S-" clipboard-context))
-                           "Send to AI (clipboard context): "
-                         "Send to AI: ")))
+         (prefix-value (when arg (prefix-numeric-value arg)))
+         (include-files-and-repo (and arg (>= prefix-value 4)))
+         (include-clipboard (and arg (>= prefix-value 16)))
+         (files-context-string (when include-files-and-repo
+                                 (ai-code--get-context-files-string)))
+         (repo-context-string (when include-files-and-repo
+                                (ai-code--format-repo-context-info)))
+         (clipboard-context (when include-clipboard
+                              (ai-code--get-clipboard-text)))
+         (prompt-label
+          (cond
+           (include-clipboard "Send to AI (files/repo/clipboard context): ")
+           (include-files-and-repo "Send to AI (files/repo context): ")
+           (t "Send to AI: "))))
     (when-let* ((prompt (ai-code-read-string prompt-label initial-input)))
-      (let ((final-prompt (if (and clipboard-context
-                                   (string-match-p "\\S-" clipboard-context))
-                              (concat prompt
-                                      "\n\nClipboard context:\n"
-                                      clipboard-context)
-                            prompt)))
+      (let ((final-prompt
+             (concat prompt
+                     (or files-context-string "")
+                     (or repo-context-string "")
+                     (when (and clipboard-context
+                                (string-match-p "\\S-" clipboard-context))
+                       (concat "\n\nClipboard context:\n"
+                               clipboard-context)))))
         (ai-code--insert-prompt final-prompt)))))
 
 ;;;###autoload
@@ -161,7 +238,11 @@ Otherwise switch to AI CLI buffer."
   (if (and current-prefix-arg
            (ai-code-backends-infra--session-buffer-p (current-buffer)))
       (quit-window)
-    (ai-code-cli-switch-to-buffer t)))
+    ;; Try with argument first; fall back to no-arg call if function doesn't accept it
+    (condition-case nil
+        (ai-code-cli-switch-to-buffer t)
+      (wrong-number-of-arguments ;; will be triggered during calling corresponding function in external backends such as claude-code-ide.el, claude-code.el, since the corresponding function doesn't have parameter
+       (ai-code-cli-switch-to-buffer)))))
 
 (defclass ai-code--use-prompt-suffix-type (transient-lisp-variable)
   ((variable :initform 'ai-code-use-prompt-suffix)
@@ -177,6 +258,31 @@ Otherwise switch to AI CLI buffer."
   :reader (lambda (_prompt _initial-input _history)
             (not ai-code-use-prompt-suffix)))
 
+(defclass ai-code--code-change-auto-test-type (transient-lisp-variable)
+  ((variable :initform 'ai-code-auto-test-type)
+   (format :initform "%k %d %v")
+   (reader :initform #'transient-lisp-variable--read-value))
+  "Selection helper for `ai-code-auto-test-type`.")
+
+(transient-define-infix ai-code--infix-select-code-change-auto-test ()
+  "Select `ai-code-auto-test-type` mode."
+  :class 'ai-code--code-change-auto-test-type
+  :key "T"
+  :description "Auto test type:"
+  :reader (lambda (_prompt _initial-input _history)
+            (let* ((choices '(("Use test after code change prompt" . test-after-change)
+                              ("Use TDD Red+Green prompt" . tdd)
+                              ("Off" . nil)))
+                   (choice (completing-read "Test after code change: "
+                                            (mapcar #'car choices)
+                                            nil t)))
+              (let ((value (cdr (assoc choice choices))))
+                (ai-code--apply-auto-test-type value)
+                (message "Auto test type set to %s; prompt suffix is now %s"
+                         (or value "off")
+                         (or ai-code-auto-test-suffix "cleared"))
+                value))))
+
 (defun ai-code--select-backend-description (&rest _)
   "Dynamic description for the Select Backend menu item.
 Shows the current backend label to the right."
@@ -187,14 +293,16 @@ Shows the current backend label to the right."
   "Transient menu for AI Code Interface interactive functions."
   ["AI Code Commands"
    ["AI CLI session"
-    ("a" "Start AI CLI" ai-code-cli-start)
+    ("a" "Start AI CLI (C-u: args)" ai-code-cli-start)
     ("R" "Resume AI CLI" ai-code-cli-resume)
     ("z" "Switch to AI CLI (C-u: hide)" ai-code-cli-switch-to-buffer-or-hide)
     ;; Use plist style to provide a dynamic description function.
     ("s" ai-code-select-backend :description ai-code--select-backend-description)
     ("u" "Install / Upgrade AI CLI" ai-code-upgrade-backend)
     ("g" "Open backend config (eg. add mcp)" ai-code-open-backend-config)
-    ("|" "Apply prompt on file" ai-code-apply-prompt-on-current-file)]
+    ("G" "Open backend repo agent file" ai-code-open-backend-agent-file)
+    ("|" "Apply prompt on file" ai-code-apply-prompt-on-current-file)
+    ]
 
    ["AI Code Actions"
     (ai-code--infix-toggle-suffix)
@@ -202,63 +310,44 @@ Shows the current backend label to the right."
     ("i" "Implement TODO (C-u: clipboard)" ai-code-implement-todo)
     ("q" "Ask question (C-u: clipboard)" ai-code-ask-question)
     ("x" "Explain code in scope" ai-code-explain)
-    ("<SPC>" "Send command (C-u: clipboard)" ai-code-send-command)
-    ("@" "Add context (C-u: clear)" ai-code-context-action)]
+    ("<SPC>" "Send command (C-u: context)" ai-code-send-command)
+    ("@" "Add context (C-u: clear)" ai-code-context-action)
+    ]
 
    ["AI Agile Development"
+    (ai-code--infix-select-code-change-auto-test)
     ("r" "Refactor Code"               ai-code-refactor-book-method)
     ("t" "Test Driven Development"     ai-code-tdd-cycle)
     ("v" "Pull or Review Code Change"  ai-code-pull-or-review-diff-file)
-    ("p" "Open prompt file" ai-code-open-prompt-file)
-    ("b" "Send prompt block to AI" ai-code-prompt-send-block)
+    ;; ("b" "Send prompt block to AI" ai-code-prompt-send-block)
     ("!" "Run Current File or Command" ai-code-run-current-file-or-shell-cmd)
-    ("B" "Build project"               ai-code-build-project)
-    ("I" "Insert function name at point" ai-code-insert-function-at-point)]
+    ("b" "Build / Test project"               ai-code-build-or-test-project)
+    ;; ("I" "Insert function name at point" ai-code-insert-function-at-point)
+    ("K" "Create or open task file" ai-code-create-or-open-task-file)
+    ("n" "Take notes from AI session region" ai-code-take-notes)
+    ]
 
    ["Other Tools"
     ("." "Init projectile and gtags" ai-code-init-project)
     ("e" "Debug exception (C-u: clipboard)" ai-code-investigate-exception)
     ("f" "Fix Flycheck errors in scope" ai-code-flycheck-fix-errors-in-scope)
     ("k" "Copy Cur File Name (C-u: full)" ai-code-copy-buffer-file-name-to-clipboard)
-    ("d" "Toggle current buffer dedicated" ai-code-toggle-current-buffer-dedicated)
+    ;; ("d" "Toggle current buffer dedicated" ai-code-toggle-current-buffer-dedicated)
+    ("o" "Open recent file (C-u: insert)" ai-code-git-repo-recent-modified-files)
     ;; ("o" "Open Clipboard file dir" ai-code-open-clipboard-file-path-as-dired)
+    ("p" "Open prompt history file" ai-code-open-prompt-file)
     ("m" "Debug python MCP server" ai-code-debug-mcp)
-    ("n" "Take notes from AI session region" ai-code-take-notes)]])
+    ("N" "Toggle notifications" ai-code-notifications-toggle)]])
 
 
 
-;; When in a special buffer (e.g., *claude-code*) and using evil-mode,
-;; pressing SPC in normal state will send the prompt.
+;; DONE: When in an ai-coding session (e.g., with -backends-infra.el code) and using evil-mode,
+;; pressing SPC in normal state will trigger ai-code-send-command.
+;; Fixed implementation moved to ai-code-backends-infra.el:
+;;   - `ai-code-backends-infra--evil-spc-command' - intercepts SPC in AI session buffers
+;;   - `ai-code-backends-infra-evil-setup' - call with (with-eval-after-load 'evil ...)
 
-;; following code is buggy
-(defvar ai-code--original-spc-command-in-evil-normal-state nil
-  "Original command for SPC in `evil-normal-state`.")
-
-(defun ai-code-spc-command-for-special-buffer-in-evil ()
-  "In special buffers (*...*), run `ai-code-send-command`.
-Otherwise, run the original command for SPC."
-  (interactive)
-  (if (and (string-prefix-p "*" (buffer-name))
-           (string-suffix-p "*" (buffer-name)))
-      (call-interactively #'ai-code-send-command)
-    (when ai-code--original-spc-command-in-evil-normal-state
-      (call-interactively ai-code--original-spc-command-in-evil-normal-state))))
-
-;;;###autoload
-(defun ai-code-evil-setup ()
-  "Setup AI Code integration with Evil mode.
-This function configures SPC key binding in Evil normal state for
-special AI Code buffers.  Call this function after Evil is loaded,
-typically in your Emacs configuration with:
-  (with-eval-after-load \\='evil (ai-code-evil-setup))"
-  (interactive)
-  (when (and (featurep 'evil) (boundp 'evil-normal-state-map))
-    (unless ai-code--original-spc-command-in-evil-normal-state
-      (setq ai-code--original-spc-command-in-evil-normal-state
-            (lookup-key evil-normal-state-map (kbd "SPC"))))
-    (when ai-code--original-spc-command-in-evil-normal-state
-      (define-key evil-normal-state-map (kbd "SPC")
-        #'ai-code-spc-command-for-special-buffer-in-evil))))
+;; (with-eval-after-load 'evil (ai-code-backends-infra-evil-setup))
 
 (provide 'ai-code)
 
